@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
-#ifndef __ALSA_HINAWA_FW_NODE_H__
-#define __ALSA_HINAWA_FW_NODE_H__
+#ifndef __ORG_KERNEL_HINAWA_FW_NODE_H__
+#define __ORG_KERNEL_HINAWA_FW_NODE_H__
 
 #include <hinawa.h>
 
@@ -8,7 +8,7 @@ G_BEGIN_DECLS
 
 #define HINAWA_TYPE_FW_NODE	(hinawa_fw_node_get_type())
 
-G_DECLARE_DERIVABLE_TYPE(HinawaFwNode, hinawa_fw_node, HINAWA, FW_NODE, GObject);
+G_DECLARE_DERIVABLE_TYPE(HinawaFwNode, hinawa_fw_node, HINAWA, FW_NODE, GObject)
 
 #define HINAWA_FW_NODE_ERROR	hinawa_fw_node_error_quark()
 
@@ -23,7 +23,7 @@ struct _HinawaFwNodeClass {
 	 *
 	 * Class closure for the [signal@FwNode::bus-update].
 	 *
-	 * Since: 1.4.
+	 * Since: 1.4
 	 */
 	void (*bus_update)(HinawaFwNode *self);
 
@@ -33,7 +33,7 @@ struct _HinawaFwNodeClass {
 	 *
 	 * Class closure for the [signal@FwNode::disconnected].
 	 *
-	 * Since: 1.4.
+	 * Since: 1.4
 	 */
 	void (*disconnected)(HinawaFwNode *self);
 
@@ -41,14 +41,15 @@ struct _HinawaFwNodeClass {
 
 HinawaFwNode *hinawa_fw_node_new(void);
 
-void hinawa_fw_node_open(HinawaFwNode *self, const gchar *path,
-			 GError **error);
+gboolean hinawa_fw_node_open(HinawaFwNode *self, const gchar *path, gint open_flag, GError **error);
 
-void hinawa_fw_node_get_config_rom(HinawaFwNode *self, const guint8 **image,
-				   gsize *length, GError **error);
+gboolean hinawa_fw_node_get_config_rom(HinawaFwNode *self, const guint8 **image, gsize *length,
+				       GError **error);
 
-void hinawa_fw_node_create_source(HinawaFwNode *self, GSource **gsrc,
-				  GError **error);
+gboolean hinawa_fw_node_read_cycle_time(HinawaFwNode *self, gint clock_id,
+					HinawaCycleTime **cycle_time, GError **error);
+
+gboolean hinawa_fw_node_create_source(HinawaFwNode *self, GSource **gsrc, GError **error);
 
 G_END_DECLS
 
