@@ -7,13 +7,9 @@
 
 /**
  * HinawaFwReq:
- * A transaction executor to a FireWire unit.
+ * A transaction executor to a node in IEEE 1394 bus.
  *
- * A HinawaFwReq supports some types of transactions in IEEE 1212. Mainly for read, write and lock
- * operations.
- *
- * This class is an application of Linux FireWire subsystem. All of operations utilize ioctl(2)
- * with subsystem specific request commands.
+ * [class@FwReq] supports all types of transactions defiend in IEEE 1212.
  */
 
 /**
@@ -27,17 +23,7 @@
  */
 G_DEFINE_QUARK(hinawa-fw-req-error-quark, hinawa_fw_req_error)
 
-/* This object has one property. */
-enum fw_req_prop_type {
-	FW_REQ_PROP_TYPE_TIMEOUT = 1,
-	FW_REQ_PROP_TYPE_COUNT,
-};
-static GParamSpec *fw_req_props[FW_REQ_PROP_TYPE_COUNT] = { NULL, };
-
-typedef struct {
-	guint timeout;
-} HinawaFwReqPrivate;
-G_DEFINE_TYPE_WITH_PRIVATE(HinawaFwReq, hinawa_fw_req, G_TYPE_OBJECT)
+G_DEFINE_TYPE(HinawaFwReq, hinawa_fw_req, G_TYPE_OBJECT)
 
 static const char *const err_labels[] = {
 	[HINAWA_FW_REQ_ERROR_CONFLICT_ERROR]	= "conflict error",
@@ -63,81 +49,38 @@ static void generate_fw_req_error_literal(GError **error, HinawaFwReqError code)
 	g_set_error_literal(error, HINAWA_FW_REQ_ERROR, code, err_labels[code]);
 }
 
-static void fw_req_get_property(GObject *obj, guint id, GValue *val,
-				GParamSpec *spec)
-{
-	HinawaFwReq *self = HINAWA_FW_REQ(obj);
-	HinawaFwReqPrivate *priv = hinawa_fw_req_get_instance_private(self);
-
-	switch (id) {
-	case FW_REQ_PROP_TYPE_TIMEOUT:
-		g_value_set_uint(val, priv->timeout);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, id, spec);
-		break;
-	}
-}
-
-static void fw_req_set_property(GObject *obj, guint id, const GValue *val,
-				GParamSpec *spec)
-{
-	HinawaFwReq *self = HINAWA_FW_REQ(obj);
-	HinawaFwReqPrivate *priv = hinawa_fw_req_get_instance_private(self);
-
-	switch (id) {
-	case FW_REQ_PROP_TYPE_TIMEOUT:
-		priv->timeout = g_value_get_uint(val);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, id, spec);
-		break;
-	}
-}
-
 enum fw_req_sig_type {
-	FW_REQ_SIG_TYPE_RESPONDED = 1,
+	FW_REQ_SIG_TYPE_RESPONDED = 0,
 	FW_REQ_SIG_TYPE_COUNT,
 };
 static guint fw_req_sigs[FW_REQ_SIG_TYPE_COUNT] = { 0 };
 
 static void hinawa_fw_req_class_init(HinawaFwReqClass *klass)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
-
-	gobject_class->get_property = fw_req_get_property;
-	gobject_class->set_property = fw_req_set_property;
-
-	/**
-	 * HinawaFwReq:timeout:
-	 *
-	 * Since: 1.4
-	 * Deprecated: 2.1: Use timeout_ms parameter of [method@FwReq.transaction_sync].
-	 */
-	fw_req_props[FW_REQ_PROP_TYPE_TIMEOUT] =
-		g_param_spec_uint("timeout", "timeout",
-				  "An elapse to expire waiting for response by ms unit.",
-				  10, G_MAXUINT,
-				  200,
-				  G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_DEPRECATED);
-
-	g_object_class_install_properties(gobject_class,
-					  FW_REQ_PROP_TYPE_COUNT,
-					  fw_req_props);
-
 	/**
 	 * HinawaFwReq::responded:
 	 * @self: A [class@FwReq].
 	 * @rcode: One of [enum@FwRcode].
+	 * @request_tstamp: The isochronous cycle at which the request subaction was sent for the
+	 *		    transaction.
+	 * @response_tstamp: The isochronous cycle at which the response subaction arrived for the
+	 *		     transaction.
 	 * @frame: (array length=frame_size)(element-type guint8): The array with elements for
-	 *	   byte data of response subaction for transaction.
+	 *	   byte data of response subaction for the transaction.
 	 * @frame_size: The number of elements of the array.
 	 *
-	 * Emitted when the unit transfers asynchronous packet as response subaction for the
+	 * Emitted when the node transfers asynchronous packet as response subaction for the
 	 * transaction and the process successfully reads the content of packet from Linux firewire
 	 * subsystem.
 	 *
-	 * Since: 2.1
+	 * The values of @request_tstamp and @response_tstamp are unsigned 16 bit integer including
+	 * higher 3 bits for three low order bits of second field and the rest 13 bits for cycle
+	 * field in the format of IEEE 1394 CYCLE_TIMER register.
+	 *
+	 * If the version of kernel ABI for Linux FireWire subsystem is less than 6, the
+	 * @request_tstamp and @response_tstamp argument has invalid value (=G_MAXUINT).
+	 *
+	 * Since: 4.0
 	 */
 	fw_req_sigs[FW_REQ_SIG_TYPE_RESPONDED] =
 		g_signal_new("responded",
@@ -145,9 +88,10 @@ static void hinawa_fw_req_class_init(HinawaFwReqClass *klass)
 			     G_SIGNAL_RUN_LAST,
 			     G_STRUCT_OFFSET(HinawaFwReqClass, responded),
 			     NULL, NULL,
-			     hinawa_sigs_marshal_VOID__ENUM_POINTER_UINT,
+			     hinawa_sigs_marshal_VOID__ENUM_UINT_UINT_POINTER_UINT,
 			     G_TYPE_NONE,
-			     3, HINAWA_TYPE_FW_RCODE, G_TYPE_POINTER, G_TYPE_UINT);
+			     5, HINAWA_TYPE_FW_RCODE, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_POINTER,
+			     G_TYPE_UINT);
 }
 
 static void hinawa_fw_req_init(HinawaFwReq *self)
@@ -161,7 +105,7 @@ static void hinawa_fw_req_init(HinawaFwReq *self)
  * Instantiate [class@FwReq] object and return the instance.
  *
  * Returns: an instance of [class@FwReq].
- * Since: 1.3.
+ * Since: 1.3
  */
 HinawaFwReq *hinawa_fw_req_new(void)
 {
@@ -169,7 +113,7 @@ HinawaFwReq *hinawa_fw_req_new(void)
 }
 
 /**
- * hinawa_fw_req_transaction_async:
+ * hinawa_fw_req_request:
  * @self: A [class@FwReq].
  * @node: A [class@FwNode].
  * @tcode: A transaction code of [enum@FwTcode].
@@ -181,30 +125,30 @@ HinawaFwReq *hinawa_fw_req_new(void)
  *	   to the array and immutable. The content of array is mutable for read and lock
  *	   transaction.
  * @frame_size: The size of array in byte unit. The value of this argument should point to the
- *		numerical number and mutable for read and lock transaction.
- * @error: A [struct@GLib.Error]. Error can be generated with two domains; Hinawa.FwNodeError and
- *	   Hinawa.FwReqError.
+ *		numeric number and mutable for read and lock transaction.
+ * @error: A [struct@GLib.Error]. Error can be generated with two domains; [error@FwNodeError] and
+ *	   [error@FwReqError].
  *
- * Execute request subaction of transactions to the given node according to given code. When the
- * response subaction arrives and read the contents, [signal@FwReq::responded] signal handler is called
- * as long as event dispatcher runs.
+ * Execute request subaction of transaction to the given node according to given code. When the
+ * response subaction arrives and running event dispatcher reads the contents,
+ * [signal@FwReq::responded] signal handler is called.
  *
- * Since: 2.1.
+ * Since: 4.0
  */
-void hinawa_fw_req_transaction_async(HinawaFwReq *self, HinawaFwNode *node,
-				     HinawaFwTcode tcode, guint64 addr, gsize length,
-				     guint8 *const *frame, gsize *frame_size,
-				     GError **error)
+gboolean hinawa_fw_req_request(HinawaFwReq *self, HinawaFwNode *node, HinawaFwTcode tcode,
+			       guint64 addr, gsize length, guint8 **frame, gsize *frame_size,
+			       GError **error)
 {
 	struct fw_cdev_send_request req = {0};
-	guint64 generation;
+	guint generation;
 	int err;
 
-	g_return_if_fail(HINAWA_IS_FW_REQ(self));
-	g_return_if_fail(length > 0);
-	g_return_if_fail(frame != NULL);
-	g_return_if_fail(frame_size != NULL && *frame_size > 0);
-	g_return_if_fail(error == NULL || *error == NULL);
+	g_return_val_if_fail(HINAWA_IS_FW_REQ(self), FALSE);
+	g_return_val_if_fail(HINAWA_IS_FW_NODE(node), FALSE);
+	g_return_val_if_fail(length > 0, FALSE);
+	g_return_val_if_fail(frame != NULL, FALSE);
+	g_return_val_if_fail(frame_size != NULL && *frame_size > 0, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
 	// Should be aligned to quadlet.
 	if (tcode == HINAWA_FW_TCODE_WRITE_QUADLET_REQUEST ||
@@ -216,14 +160,14 @@ void hinawa_fw_req_transaction_async(HinawaFwReq *self, HinawaFwNode *node,
 	    tcode == HINAWA_FW_TCODE_LOCK_BOUNDED_ADD ||
 	    tcode == HINAWA_FW_TCODE_LOCK_WRAP_ADD ||
 	    tcode == HINAWA_FW_TCODE_LOCK_VENDOR_DEPENDENT)
-		g_return_if_fail(!(addr & 0x3) && !(length & 0x3));
+		g_return_val_if_fail(!(addr & 0x3) && !(length & 0x3), FALSE);
 
 	// Should have enough space for read/written data.
 	if (tcode == HINAWA_FW_TCODE_READ_QUADLET_REQUEST ||
 	    tcode == HINAWA_FW_TCODE_READ_BLOCK_REQUEST ||
 	    tcode == HINAWA_FW_TCODE_WRITE_QUADLET_REQUEST ||
 	    tcode == HINAWA_FW_TCODE_WRITE_BLOCK_REQUEST) {
-		g_return_if_fail(*frame_size >= length);
+		g_return_val_if_fail(*frame_size >= length, FALSE);
 	} else if (tcode == HINAWA_FW_TCODE_LOCK_MASK_SWAP ||
 		   tcode == HINAWA_FW_TCODE_LOCK_COMPARE_SWAP ||
 		   tcode == HINAWA_FW_TCODE_LOCK_FETCH_ADD ||
@@ -231,14 +175,14 @@ void hinawa_fw_req_transaction_async(HinawaFwReq *self, HinawaFwNode *node,
 		   tcode == HINAWA_FW_TCODE_LOCK_BOUNDED_ADD ||
 		   tcode == HINAWA_FW_TCODE_LOCK_WRAP_ADD ||
 		   tcode == HINAWA_FW_TCODE_LOCK_VENDOR_DEPENDENT) {
-		g_return_if_fail(*frame_size >= length * 2);
+		g_return_val_if_fail(*frame_size >= length * 2, FALSE);
 		length *= 2;
 	} else {
 		// Not supported due to no test.
-		g_return_if_reached();
+		g_return_val_if_reached(FALSE);
 	}
 
-	// Get unit properties.
+	// Get node property.
 	g_object_get(G_OBJECT(node), "generation", &generation, NULL);
 
 	// Setup a transaction structure.
@@ -255,18 +199,23 @@ void hinawa_fw_req_transaction_async(HinawaFwReq *self, HinawaFwNode *node,
 	err = hinawa_fw_node_ioctl(node, FW_CDEV_IOC_SEND_REQUEST, &req, error);
 	if (*error == NULL && err > 0)
 		generate_fw_req_error_with_errno(error, HINAWA_FW_REQ_ERROR_SEND_ERROR, err);
+
+	return err >= 0;
 }
 
 struct waiter {
 	guint rcode;
+	guint request_tstamp;
+	guint response_tstamp;
 	guint8 *frame;
 	gsize length;
 	GCond cond;
 	GMutex mutex;
 };
 
-static void handle_responded_signal(HinawaFwReq *self, HinawaFwRcode rcode, const guint8 *frame,
-				    guint frame_size, gpointer user_data)
+static void handle_responded_signal(HinawaFwReq *self, HinawaFwRcode rcode, guint request_tstamp,
+				    guint response_tstamp, const guint8 *frame, guint frame_size,
+				    gpointer user_data)
 {
 	struct waiter *w = (struct waiter *)user_data;
 
@@ -274,6 +223,9 @@ static void handle_responded_signal(HinawaFwReq *self, HinawaFwRcode rcode, cons
 
 	w->rcode = rcode;
 
+	w->request_tstamp = request_tstamp;
+	w->response_tstamp = response_tstamp;
+
 	w->length = MIN(w->length, frame_size);
 	memcpy(w->frame, frame, w->length);
 
@@ -284,7 +236,7 @@ static void handle_responded_signal(HinawaFwReq *self, HinawaFwRcode rcode, cons
 }
 
 /**
- * hinawa_fw_req_transaction_sync:
+ * hinawa_fw_req_transaction_with_tstamp:
  * @self: A [class@FwReq].
  * @node: A [class@FwNode].
  * @tcode: A transaction code of [enum@FwTcode].
@@ -297,47 +249,58 @@ static void handle_responded_signal(HinawaFwReq *self, HinawaFwRcode rcode, cons
  *	   transaction.
  * @frame_size: The size of array in byte unit. The value of this argument should point to the
  *		numeric number and mutable for read and lock transaction.
- * @timeout_ms: The timeout to wait for response subaction of the transaction since request
+ * @tstamp: (array fixed-size=2)(out caller-allocates): The array with two elements for time stamps.
+ *	    The first element is for the isochronous cycle at which the request subaction was sent.
+ *	    The second element is for the isochronous cycle at which the response subaction arrived.
+ * @timeout_ms: The timeout to wait for the response subaction of transaction since the request
  *		subaction is initiated, in milliseconds.
- * @error: A [struct@GLib.Error]. Error can be generated with two domains; Hinawa.FwNodeError and
- *	   Hinawa.FwReqError.
+ * @error: A [struct@GLib.Error]. Error can be generated with two domains; [error@FwNodeError] and
+ *	   [error@FwReqError].
  *
  * Execute request subaction of transaction to the given node according to given code, then wait
- * for response subaction within the given timeout. The [property@FwReq:timeout] property of
- * instance is ignored.
+ * for response subaction within the given timeout.
  *
- * Since: 2.1.
+ * Each value of @tstamp is unsigned 16 bit integer including higher 3 bits for three low order bits
+ * of second field and the rest 13 bits for cycle field in the format of IEEE 1394 CYCLE_TIMER register.
+ *
+ * If the version of kernel ABI for Linux FireWire subsystem is less than 6, each element of @tstamp
+ * has invalid value (=G_MAXUINT).
+ *
+ * Returns: TRUE if the overall operation finishes successfully, otherwise FALSE.
+ * Since: 2.6
  */
-void hinawa_fw_req_transaction_sync(HinawaFwReq *self, HinawaFwNode *node,
-			       HinawaFwTcode tcode, guint64 addr, gsize length,
-			       guint8 *const *frame, gsize *frame_size, guint timeout_ms,
-			       GError **error)
+gboolean hinawa_fw_req_transaction_with_tstamp(HinawaFwReq *self, HinawaFwNode *node,
+					       HinawaFwTcode tcode, guint64 addr, gsize length,
+					       guint8 **frame, gsize *frame_size, guint tstamp[2],
+					       guint timeout_ms, GError **error)
 {
-	gulong handler_id;
 	struct waiter w;
+	gulong handler_id;
 	guint64 expiration;
 
-	g_return_if_fail(HINAWA_IS_FW_REQ(self));
+	g_return_val_if_fail(HINAWA_IS_FW_REQ(self), FALSE);
+	g_return_val_if_fail(HINAWA_IS_FW_NODE(node), FALSE);
+	g_return_val_if_fail(tstamp != NULL, FALSE);
 
 	// This predicates against suprious wakeup.
 	w.rcode = G_MAXUINT;
+	w.request_tstamp = G_MAXUINT;
+	w.response_tstamp = G_MAXUINT;
 	w.frame = *frame;
 	w.length = *frame_size;
 	g_cond_init(&w.cond);
 	g_mutex_init(&w.mutex);
 
-        handler_id = g_signal_connect(self, "responded", (GCallback)handle_responded_signal, &w);
+        handler_id = g_signal_connect(self, "responded", G_CALLBACK(handle_responded_signal), &w);
 
 	// Timeout is set in advance as a parameter of this object.
 	expiration = g_get_monotonic_time() + timeout_ms * G_TIME_SPAN_MILLISECOND;
 
 	g_object_ref(node);
-	hinawa_fw_req_transaction_async(self, node, tcode, addr, length, frame, frame_size,
-					error);
-	if (*error != NULL) {
+	if (!hinawa_fw_req_request(self, node, tcode, addr, length, frame, frame_size, error)) {
 		g_signal_handler_disconnect(self, handler_id);
 		g_object_unref(node);
-		return;
+		return FALSE;
 	}
 
 	g_mutex_lock(&w.mutex);
@@ -356,13 +319,16 @@ void hinawa_fw_req_transaction_sync(HinawaFwReq *self, HinawaFwNode *node,
 
 	if (w.rcode == G_MAXUINT) {
 		generate_fw_req_error_literal(error, HINAWA_FW_REQ_ERROR_CANCELLED);
-		return;
+		return FALSE;
 	}
 
+	tstamp[0] = w.request_tstamp;
+	tstamp[1] = w.response_tstamp;
+
 	switch (w.rcode) {
 	case RCODE_COMPLETE:
 		*frame_size = w.length;
-		break;
+		return TRUE;
 	case RCODE_CONFLICT_ERROR:
 	case RCODE_DATA_ERROR:
 	case RCODE_TYPE_ERROR:
@@ -373,10 +339,10 @@ void hinawa_fw_req_transaction_sync(HinawaFwReq *self, HinawaFwNode *node,
 	case RCODE_GENERATION:
 	case RCODE_NO_ACK:
 		generate_fw_req_error_literal(error, (HinawaFwReqError)w.rcode);
-		break;
+		return FALSE;
 	default:
 		generate_fw_req_error_literal(error, HINAWA_FW_REQ_ERROR_INVALID);
-		break;
+		return FALSE;
 	}
 }
 
@@ -394,35 +360,44 @@ void hinawa_fw_req_transaction_sync(HinawaFwReq *self, HinawaFwNode *node,
  *	   transaction.
  * @frame_size: The size of array in byte unit. The value of this argument should point to the
  *		numerical number and mutable for read and lock transaction.
- * @error: A [struct@GLib.Error]. Error can be generated with two domains; Hinawa.FwNodeError and
- *	   Hinawa.FwReqError.
+ * @timeout_ms: The timeout to wait for response subaction of the transaction since request
+ *		subaction is initiated, in milliseconds.
+ * @error: A [struct@GLib.Error]. Error can be generated with two domains; [error@FwNodeError] and
+ *	   [error@FwReqError].
  *
  * Execute request subaction of transaction to the given node according to given code, then wait
- * for response subaction within the value of timeout argument.
+ * for response subaction within the value of timeout argument. The function is a thin wrapper to
+ * [method@FwReq.transaction_with_tstamp].
  *
- * Since: 1.4
- * Deprecated: 2.1: Use [method@FwReq.transaction_sync] instead.
+ * Returns: TRUE if the overall operation finishes successfully, otherwise FALSE.
+ *
+ * Since: 4.0
  */
-void hinawa_fw_req_transaction(HinawaFwReq *self, HinawaFwNode *node,
+gboolean hinawa_fw_req_transaction(HinawaFwReq *self, HinawaFwNode *node,
 			       HinawaFwTcode tcode, guint64 addr, gsize length,
-			       guint8 *const *frame, gsize *frame_size,
+			       guint8 **frame, gsize *frame_size, guint timeout_ms,
 			       GError **error)
 {
-	HinawaFwReqPrivate *priv;
+	guint tstamp[2];
 
-	g_return_if_fail(HINAWA_IS_FW_REQ(self));
-	priv = hinawa_fw_req_get_instance_private(self);
-
-	hinawa_fw_req_transaction_sync(self, node, tcode, addr, length, frame, frame_size,
-				       priv->timeout, error);
+	return hinawa_fw_req_transaction_with_tstamp(self, node, tcode, addr, length,
+						     frame, frame_size, tstamp, timeout_ms, error);
 }
 
 // NOTE: For HinawaFwNode, internal.
-void hinawa_fw_req_handle_response(HinawaFwReq *self,
-				   struct fw_cdev_event_response *event)
+void hinawa_fw_req_handle_response(HinawaFwReq *self, const struct fw_cdev_event_response *event)
 {
 	g_return_if_fail(HINAWA_IS_FW_REQ(self));
 
-	g_signal_emit(self, fw_req_sigs[FW_REQ_SIG_TYPE_RESPONDED], 0,
-		      event->rcode, event->data, event->length);
+	g_signal_emit(self, fw_req_sigs[FW_REQ_SIG_TYPE_RESPONDED], 0, event->rcode, G_MAXUINT,
+		      G_MAXUINT, event->data, event->length);
+}
+
+// NOTE: For HinawaFwNode, internal.
+void hinawa_fw_req_handle_response2(HinawaFwReq *self, const struct fw_cdev_event_response2 *event)
+{
+	g_return_if_fail(HINAWA_IS_FW_REQ(self));
+
+	g_signal_emit(self, fw_req_sigs[FW_REQ_SIG_TYPE_RESPONDED], 0, event->rcode,
+		      event->request_tstamp, event->response_tstamp, event->data, event->length);
 }
